/* 
 * Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
 *
 * This file is part of Rast.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 *
 * Linking the Rast statically or dynamically with other modules is
 * making a combined work based on the Rast.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of the Rast give
 * you permission to link the Rast with independent modules that
 * communicate with the Rast solely through the the Apache Portable
 * Runtime interface, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * combined work under terms of your choice, provided that every
 * copy of the combined work is accompanied by a complete copy of
 * the source code of the Rast (the version of the Rast used to
 * produce the combined work), being distributed under the terms of
 * the GNU General Public License plus this exception.  An
 * independent module is a module which is not derived from or based
 * on the Rast.
 *
 * Note that people who make modified versions of the Rast are not
 * obligated to grant this special exception for their modified
 * versions; it is their choice whether to do so.  The GNU General
 * Public License gives permission to release a modified version
 * without this exception; this exception also makes it possible to
 * release a modified version which carries forward this exception.
 */

#include <stdlib.h>

#include "rast/xmlrpc_client.h"

static int memory_error(int retcode);
static void print_error(rast_error_t *error);

int
main(int argc, char **argv)
{
    apr_pool_t *pool;
    char *server_url, *query;
    rast_db_t *db;
    rast_search_option_t *options;
    rast_result_t *result;
    rast_error_t *error;
    int i;
    const char *properties[] = {
        "filename",
        "title",
        "last_modified",
        "filesize"
    };

    if (argc < 3) {
        fprintf(stderr, "usage: %s <server_url> <query>\n", argv[0]);
        return 1;
    }
    server_url = argv[1];
    query = argv[2];

    apr_initialize();
    atexit(apr_terminate);
    error = rast_xmlrpc_client_initialize();
    if (error != RAST_OK) {
        return 1;
    }
    atexit(rast_xmlrpc_client_finalize);

    apr_pool_create_ex(&pool, NULL, memory_error, NULL);

    error = rast_xmlrpc_client_open(&db, server_url, RAST_DB_RDONLY, NULL,
                                    pool);
    if (error != RAST_OK) {
        print_error(error);
        return 1;
    }

    options = rast_search_option_create(pool);
    options->num_items = 10;
    options->need_summary = 1;
    options->summary_nchars = 200;
    options->properties = properties;
    options->num_properties = 4;

    error = rast_db_search(db, query, options, &result, pool);
    if (error != RAST_OK) {
        rast_db_close(db);
        print_error(error);
        return 1;
    }

    printf("hit_count: %d\n", result->hit_count);
    puts("");
    for (i = 0; i < result->num_items; i++) {
        printf("doc_id: %d\n", result->items[i]->doc_id);
        printf("score: %d\n", result->items[i]->score);
        printf("filename: %s\n",
               rast_value_string(result->items[i]->properties + 0));
        printf("title: %s\n",
               rast_value_string(result->items[i]->properties + 1));
        printf("last_modified: %s\n",
               rast_value_date(result->items[i]->properties + 2));
        printf("filesize: %u\n",
               rast_value_uint(result->items[i]->properties + 3));
        puts("summary:");
        fwrite(result->items[i]->summary, 1, result->items[i]->summary_nbytes,
               stdout);
        puts("");
        puts("");
    }

    rast_db_close(db);

    return 0;
}

static int
memory_error(int retcode)
{
    abort();
    return -1;  /* prevent compiler warnings */
}

static void
print_error(rast_error_t *error)
{
#ifdef RAST_DEBUG
    fprintf(stderr, "%s:%d: %s\n", error->file, error->line, error->message);
#else
    fprintf(stderr, "error: %s\n", error->message);
#endif
    rast_error_destroy(error);
}

/* vim: set filetype=c sw=4 expandtab : */
