/* 
 * Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
 *
 * This file is part of Rast.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 *
 * Linking the Rast statically or dynamically with other modules is
 * making a combined work based on the Rast.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of the Rast give
 * you permission to link the Rast with independent modules that
 * communicate with the Rast solely through the the Apache Portable
 * Runtime interface, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * combined work under terms of your choice, provided that every
 * copy of the combined work is accompanied by a complete copy of
 * the source code of the Rast (the version of the Rast used to
 * produce the combined work), being distributed under the terms of
 * the GNU General Public License plus this exception.  An
 * independent module is a module which is not derived from or based
 * on the Rast.
 *
 * Note that people who make modified versions of the Rast are not
 * obligated to grant this special exception for their modified
 * versions; it is their choice whether to do so.  The GNU General
 * Public License gives permission to release a modified version
 * without this exception; this exception also makes it possible to
 * release a modified version which carries forward this exception.
 */

#include <stdlib.h>
#include <stdarg.h>

#include <apr_errno.h>
#include <apr_strings.h>

#include <db.h>

#include "rast/config.h"
#include "rast/rast.h"
#include "rast/error.h"

#ifdef RAST_DEBUG
#undef rast_error_vcreate
#undef rast_error_create
#endif

static const char *
rast_strerror(int code)
{
    switch (code) {
    case RAST_ERROR_NOT_IMPLEMENTED:
        return "not implemented";
    case RAST_ERROR_CLOSED:
        return "already closed";
    case RAST_ERROR_EOF:
        return "unexpected eof";
    case RAST_ERROR_BAD_DB:
        return "bad db";
    case RAST_ERROR_INVALID_ARGUMENT:
        return "invaild argument";
    case RAST_ERROR_INVALID_QUERY:
        return "invaild query";
    case RAST_ERROR_EMPTY_QUERY:
        return "empty query";
    default:
        return "unknown error";
    }
}

rast_error_t *
rast_error_vcreate(rast_error_type_t type, int code,
                   const char *fmt, va_list ap)
{
    apr_pool_t *pool;
    rast_error_t *error;
    char *message;

    pool = rast_pool_create(NULL);
    error = (rast_error_t *) apr_palloc(pool, sizeof(rast_error_t));
    error->pool = pool;
    error->type = type;
    error->code = code;
    if (fmt != NULL) {
        error->message = apr_pvsprintf(pool, fmt, ap);
    }
    else {
        switch (error->type) {
        case RAST_ERROR_TYPE_RAST:
            error->message = rast_strerror(error->code);
            break;
        case RAST_ERROR_TYPE_APR:
            message = (char *) apr_palloc(error->pool, RAST_BUFSIZ);
            apr_strerror(error->code, message, RAST_BUFSIZ);
            error->message = message;
            break;
        case RAST_ERROR_TYPE_BDB:
            error->message = db_strerror(error->code);
            break;
        default:
            error->message = "unknown error";
            break;
        }
    }

    return error;
}

rast_error_t *
rast_error_create(rast_error_type_t type, int code, const char *fmt, ...)
{
    rast_error_t *error;
    va_list ap;

    va_start(ap, fmt);
    error = rast_error_vcreate(type, code, fmt, ap);
    va_end(ap);

    return error;
}

#ifdef RAST_DEBUG
rast_error_t *
rast_error_vcreate_debug(rast_error_type_t type, int code,
                         const char *file, int line,
                         const char *fmt, va_list ap)
{
    rast_error_t *error;

    error = rast_error_vcreate(type, code, fmt, ap);
    error->file = file;
    error->line = line;
    return error;
}

rast_error_t *
rast_error_create_debug(rast_error_type_t type, int code,
                        const char *file, int line, const char *fmt, ...)
{
    rast_error_t *error;
    va_list ap;

    va_start(ap, fmt);
    error = rast_error_vcreate_debug(type, code, file, line, fmt, ap);
    va_end(ap);
    return error;
}

#endif

void
rast_error_destroy(rast_error_t *error)
{
    apr_pool_destroy(error->pool);
}

/* vim: set filetype=c sw=4 expandtab : */
